@@ -1,17 +1,17 @@
-import re
+import json
 import tempfile
 from time import sleep
 from typing import Any, Callable, Collection, Dict, List, Optional, Type, TypeVar
 
-from metaphor.models.crawler_run_metadata import Platform
-
-try:
-    import msal
-except ImportError:
-    print("Please install metaphor[power_bi] extra\n")
-    raise
-
 import requests
+from pydantic import BaseModel, parse_obj_as
+
+from metaphor.common.entity_id import EntityId
+from metaphor.common.event_util import ENTITY_TYPES
+from metaphor.common.extractor import BaseExtractor
+from metaphor.common.logger import get_logger
+from metaphor.common.utils import chunks, unique_list
+from metaphor.models.crawler_run_metadata import Platform
 from metaphor.models.metadata_change_event import (
     Chart,
     ChartType,
@@ -20,34 +20,42 @@ from metaphor.models.metadata_change_event import (
     DashboardLogicalID,
     DashboardPlatform,
     DashboardUpstream,
-    DataPlatform,
     EntityType,
 )
+from metaphor.models.metadata_change_event import PowerBIApp as PbiApp
 from metaphor.models.metadata_change_event import PowerBIColumn as PbiColumn
 from metaphor.models.metadata_change_event import PowerBIDashboardType
 from metaphor.models.metadata_change_event import (
     PowerBIDataset as VirtualViewPowerBIDataset,
 )
-from metaphor.models.metadata_change_event import PowerBIDatasetTable
+from metaphor.models.metadata_change_event import PowerBIDatasetTable, PowerBIInfo
 from metaphor.models.metadata_change_event import PowerBIMeasure as PbiMeasure
+from metaphor.models.metadata_change_event import PowerBIWorkspace as PbiWorkspace
 from metaphor.models.metadata_change_event import (
     SourceInfo,
     VirtualView,
     VirtualViewLogicalID,
     VirtualViewType,
 )
-from pydantic import BaseModel, parse_obj_as
-
-from metaphor.common.entity_id import EntityId, dataset_fullname, to_dataset_entity_id
-from metaphor.common.event_util import ENTITY_TYPES
-from metaphor.common.extractor import BaseExtractor
-from metaphor.common.logger import get_logger
-from metaphor.common.utils import chunks, unique_list
 from metaphor.power_bi.config import PowerBIRunConfig
+from metaphor.power_bi.power_query_parser import PowerQueryParser
+
+try:
+    import msal
+except ImportError:
+    print("Please install metaphor[power_bi] extra\n")
+    raise
+
 
 logger = get_logger(__name__)
 
 
+class PowerBIApp(BaseModel):
+    id: str
+    name: str
+    workspaceId: str
+
+
 class PowerBIDataSource(BaseModel):
     datasourceType: str
     datasourceId: str
@@ -92,7 +100,7 @@ class PowerBITile(BaseModel):
 
 class PowerBITableColumn(BaseModel):
     name: str
-    datatype: str = "unknown"
+    dataType: str = "unknown"
 
 
 class PowerBITableMeasure(BaseModel):
@@ -121,12 +129,14 @@ class WorkspaceInfoDataset(BaseModel):
 
 
 class WorkspaceInfoDashboard(BaseModel):
-    displayName: str
     id: str
+    appId: Optional[str] = None
+    displayName: str
 
 
 class WorkspaceInfoReport(BaseModel):
     id: str
+    appId: Optional[str] = None
     name: str
     datasetId: Optional[str] = None
     description: str = ""
@@ -179,6 +189,13 @@ class PowerBIClient:
             url, List[PowerBIWorkspace], transform_response=lambda r: r.json()["value"]
         )
 
+    def get_apps(self) -> List[PowerBIApp]:
+        # https://docs.microsoft.com/en-us/rest/api/power-bi/admin/apps-get-apps-as-admin
+        url = f"{self.API_ENDPOINT}/admin/apps?$top=5000"
+        return self._call_get(
+            url, List[PowerBIApp], transform_response=lambda r: r.json()["value"]
+        )
+
     def get_tiles(self, dashboard_id: str) -> List[PowerBITile]:
         # https://docs.microsoft.com/en-us/rest/api/power-bi/admin/dashboards-get-tiles-as-admin
         url = f"{self.API_ENDPOINT}/admin/dashboards/{dashboard_id}/tiles"
@@ -290,6 +307,9 @@ class PowerBIClient:
         ), "Authentication error. Please enable read-only Power BI admin API access for the app."
 
         assert result.status_code == 200, f"GET {url} failed, {result.content.decode()}"
+
+        logger.debug(f"Response from {url}:")
+        logger.debug(json.dumps(result.json(), indent=2))
         return parse_obj_as(type_, transform_response(result))
 
 
@@ -321,19 +341,21 @@ class PowerBIExtractor(BaseExtractor):
 
         logger.info(f"Process {len(config.workspaces)} workspaces: {config.workspaces}")
 
+        apps = self.client.get_apps()
+        app_map = {app.id: app for app in apps}
+
         for workspace_ids in chunks(
             config.workspaces, PowerBIClient.MAX_WORKSPACES_PER_SCAN
         ):
-            for info in self.client.get_workspace_info(workspace_ids):
-                workspace_id = info.id
+            for workspace in self.client.get_workspace_info(workspace_ids):
                 logger.info(
-                    f"Fetching metadata from Power BI workspace ID: {workspace_id}"
+                    f"Fetching metadata from Power BI workspace ID: {workspace.id}"
                 )
 
                 try:
-                    self.map_wi_datasets_to_virtual_views(workspace_id, info.datasets)
-                    self.map_wi_reports_to_dashboard(workspace_id, info.reports)
-                    self.map_wi_dashboards_to_dashboard(workspace_id, info.dashboards)
+                    self.map_wi_datasets_to_virtual_views(workspace)
+                    self.map_wi_reports_to_dashboard(workspace, app_map)
+                    self.map_wi_dashboards_to_dashboard(workspace, app_map)
                 except Exception as e:
                     logger.exception(e)
 
@@ -343,20 +365,18 @@ class PowerBIExtractor(BaseExtractor):
 
         return entities
 
-    def map_wi_datasets_to_virtual_views(
-        self, workspace_id: str, wi_datasets: List[WorkspaceInfoDataset]
-    ) -> None:
+    def map_wi_datasets_to_virtual_views(self, workspace: WorkspaceInfo) -> None:
 
-        dataset_map = {d.id: d for d in self.client.get_datasets(workspace_id)}
+        dataset_map = {d.id: d for d in self.client.get_datasets(workspace.id)}
 
-        for wds in wi_datasets:
-            source_datasets = []
+        for wds in workspace.datasets:
+            sources = []
             tables = []
             for table in wds.tables:
                 tables.append(
                     PowerBIDatasetTable(
                         columns=[
-                            PbiColumn(field=c.name, type=c.datatype)
+                            PbiColumn(field=c.name, type=c.dataType)
                             for c in table.columns
                         ],
                         measures=[
@@ -368,22 +388,13 @@ class PowerBIExtractor(BaseExtractor):
                 )
 
                 for source in table.source:
-                    power_query_text = source["expression"]
+                    sources.append(source["expression"])
 
-                    if "Value.NativeQuery(" in power_query_text:
-                        logger.warning(
-                            f"Skip {table.name}, because it is created from Custom SQL"
-                        )
-                        continue
-                    try:
-                        entity_id = str(
-                            PowerBIExtractor.parse_power_query(power_query_text)
-                        )
-                        source_datasets.append(entity_id)
-                    except AssertionError:
-                        logger.warning(
-                            f"Parsing upstream fail, exp: {power_query_text}"
-                        )
+            source_datasets = [
+                str(dataset)
+                for source in sources
+                for dataset in PowerQueryParser.parse_source_datasets(source)
+            ]
 
             ds = dataset_map.get(wds.id, None)
             if ds is None:
@@ -400,18 +411,19 @@ class PowerBIExtractor(BaseExtractor):
                     url=ds.webUrl,
                     source_datasets=unique_list(source_datasets),
                     description=wds.description,
+                    workspace=PbiWorkspace(id=workspace.id, name=workspace.name),
                 ),
             )
 
             self._virtual_views[wds.id] = virtual_view
 
     def map_wi_reports_to_dashboard(
-        self, workspace_id: str, wi_reports: List[WorkspaceInfoReport]
+        self, workspace: WorkspaceInfo, app_map: Dict[str, PowerBIApp]
     ) -> None:
 
-        report_map = {r.id: r for r in self.client.get_reports(workspace_id)}
+        report_map = {r.id: r for r in self.client.get_reports(workspace.id)}
 
-        for wi_report in wi_reports:
+        for wi_report in workspace.reports:
             if wi_report.datasetId is None:
                 logger.warn(f"Skipping report without datasetId: {wi_report.id}")
                 continue
@@ -428,6 +440,10 @@ class PowerBIExtractor(BaseExtractor):
                 logger.warn(f"Skipping invalid report {wi_report.id}")
                 continue
 
+            pbi_info = self._make_power_bi_info(
+                PowerBIDashboardType.REPORT, workspace, wi_report.appId, app_map
+            )
+
             dashboard = Dashboard(
                 logical_id=DashboardLogicalID(
                     dashboard_id=wi_report.id,
@@ -435,8 +451,8 @@ class PowerBIExtractor(BaseExtractor):
                 ),
                 dashboard_info=DashboardInfo(
                     description=wi_report.description,
-                    power_bi_dashboard_type=PowerBIDashboardType.REPORT,
                     title=wi_report.name,
+                    power_bi=pbi_info,
                 ),
                 source_info=SourceInfo(
                     main_url=report.webUrl,
@@ -446,12 +462,12 @@ class PowerBIExtractor(BaseExtractor):
             self._dashboards[wi_report.id] = dashboard
 
     def map_wi_dashboards_to_dashboard(
-        self, workspace_id: str, wi_dashboards: List[WorkspaceInfoDashboard]
+        self, workspace: WorkspaceInfo, app_map: Dict[str, PowerBIApp]
     ) -> None:
 
-        dashboard_map = {d.id: d for d in self.client.get_dashboards(workspace_id)}
+        dashboard_map = {d.id: d for d in self.client.get_dashboards(workspace.id)}
 
-        for wi_dashboard in wi_dashboards:
+        for wi_dashboard in workspace.dashboards:
             tiles = self.client.get_tiles(wi_dashboard.id)
             upstream = []
             for tile in tiles:
@@ -475,6 +491,10 @@ class PowerBIExtractor(BaseExtractor):
                 logger.warn(f"Skipping invalid dashboard {wi_dashboard.id}")
                 continue
 
+            pbi_info = self._make_power_bi_info(
+                PowerBIDashboardType.DASHBOARD, workspace, wi_dashboard.appId, app_map
+            )
+
             dashboard = Dashboard(
                 logical_id=DashboardLogicalID(
                     dashboard_id=wi_dashboard.id,
@@ -483,7 +503,7 @@ class PowerBIExtractor(BaseExtractor):
                 dashboard_info=DashboardInfo(
                     title=wi_dashboard.displayName,
                     charts=self.transform_tiles_to_charts(tiles),
-                    power_bi_dashboard_type=PowerBIDashboardType.DASHBOARD,
+                    power_bi=pbi_info,
                 ),
                 source_info=SourceInfo(
                     main_url=pbi_dashboard.webUrl,
@@ -492,57 +512,25 @@ class PowerBIExtractor(BaseExtractor):
             )
             self._dashboards[wi_dashboard.id] = dashboard
 
-    @staticmethod
-    def parse_power_query(expression: str) -> EntityId:
-        lines = expression.split("\n")
-        platform_pattern = re.compile(r"Source = (\w+).")
-        match = platform_pattern.search(lines[1])
-        assert match, "Can't parse platform from power query expression."
-        platform_str = match.group(1)
-
-        field_pattern = re.compile(r'{\[Name="([\w\-]+)"(.*)\]}')
-
-        def get_field(text: str) -> str:
-            match = field_pattern.search(text)
-            assert match, "Can't parse field from power query expression"
-            return match.group(1)
-
-        account = None
-        if platform_str == "AmazonRedshift":
-            platform = DataPlatform.REDSHIFT
-            db_pattern = re.compile(r"Source = (\w+).Database\((.*)\),$")
-            match = db_pattern.search(lines[1])
-            assert (
-                match
-            ), "Can't parse AmazonRedshift database from power query expression"
-
-            db = match.group(2).split(",")[1].replace('"', "")
-            schema = get_field(lines[2])
-            table = get_field(lines[3])
-        elif platform_str == "Snowflake":
-            platform = DataPlatform.SNOWFLAKE
-            account_pattern = re.compile(r'Snowflake.Databases\("([\w\-\.]+)"')
-
-            # remove trailing snowflakecomputing.com
-            match = account_pattern.search(lines[1])
-            assert match, "Can't parse Snowflake account from power query expression"
-
-            account = ".".join(match.group(1).split(".")[:-2])
-            db = get_field(lines[2])
-            schema = get_field(lines[3])
-            table = get_field(lines[4])
-        elif platform_str == "GoogleBigQuery":
-            platform = DataPlatform.BIGQUERY
-            db = get_field(lines[2])
-            schema = get_field(lines[3])
-            table = get_field(lines[4])
-        else:
-            raise AssertionError(f"Unknown platform ${platform_str}")
-
-        return to_dataset_entity_id(
-            dataset_fullname(db, schema, table), platform, account
+    def _make_power_bi_info(
+        self,
+        type: PowerBIDashboardType,
+        workspace: WorkspaceInfo,
+        app_id: Optional[str],
+        app_map: Dict[str, PowerBIApp],
+    ) -> PowerBIInfo:
+        pbi_info = PowerBIInfo(
+            power_bi_dashboard_type=type,
+            workspace=PbiWorkspace(id=workspace.id, name=workspace.name),
         )
 
+        if app_id is not None:
+            app = app_map.get(app_id)
+            if app is not None:
+                pbi_info.app = PbiApp(id=app.id, name=app.name)
+
+        return pbi_info
+
     @staticmethod
     def transform_tiles_to_charts(tiles: List[PowerBITile]) -> List[Chart]:
         return [
