@@ -21,7 +21,6 @@ except ImportError:
 
 from metaphor.common.extractor import BaseExtractor
 from metaphor.models.metadata_change_event import (
-    AspectType,
     DataPlatform,
     Dataset,
     DatasetLogicalID,
@@ -227,10 +226,12 @@ class SnowflakeExtractor(BaseExtractor):
             dataset = self._datasets[fullname]
             assert dataset.schema.sql_schema is not None
 
+            # Timestamp is in nanosecond.
+            # See https://docs.snowflake.com/en/sql-reference/functions/system_last_change_commit_time.html
             timestamp = results[f"UPDATED_{fullname}"]
             if timestamp > 0:
                 dataset.statistics.last_updated = datetime.utcfromtimestamp(
-                    timestamp / 1000
+                    timestamp / 1000000000
                 ).replace(tzinfo=timezone.utc)
 
     def _fetch_unique_keys(self, cursor: SnowflakeCursor, database: str) -> None:
@@ -336,16 +337,17 @@ class SnowflakeExtractor(BaseExtractor):
             main_url=SnowflakeExtractor.build_table_url(self.account, full_name)
         )
 
-        dataset.schema = DatasetSchema()
-        dataset.schema.aspect_type = AspectType.DATASET_SCHEMA
-        dataset.schema.schema_type = SchemaType.SQL
-        dataset.schema.description = comment
-        dataset.schema.fields = []
-        dataset.schema.sql_schema = SQLSchema()
-        dataset.schema.sql_schema.materialization = (
-            MaterializationType.VIEW
-            if table_type == SnowflakeTableType.VIEW.value
-            else MaterializationType.TABLE
+        dataset.schema = DatasetSchema(
+            schema_type=SchemaType.SQL,
+            description=comment,
+            fields=[],
+            sql_schema=SQLSchema(
+                materialization=(
+                    MaterializationType.VIEW
+                    if table_type == SnowflakeTableType.VIEW.value
+                    else MaterializationType.TABLE
+                )
+            ),
         )
 
         dataset.statistics = DatasetStatistics()
